package br.usp.ime.coed.DEFSupport;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Set;

import javax.swing.JPanel;

/**
 * This panel is proper for adding and representing DrawingElement objects.
 * Beside this, its a DrawingElementFinder view, i.e., this class
 * uses all the functionality that DrawingElementFinder provides.
 *
 * Just subclass, populate, and mount it anywhere.
 */
public abstract class AbstractDrawingElementFinderClient extends JPanel
///////////////////////////////////////////////////////////////////////
{

  private DrawingElementSelector selector;

  /**
   * Manages any DrawingElement selection created by the user.
   */
  protected DrawingElementSelection  selection;
  /**
   * Is responsible for highlighting DrawingElement objects
   * (this happens in situations defined by the
   *  DrawingElementFinder behaviour of this view).
   */
  private DrawingElementHighlighter highlighter;
  /**
   * Responsible for showing up a selection in graphical mode.
   */
  private SelectionDisplayer selectionDisplayer;
  /**
   * Manages the dragging process of DrawingElement objects.
   */
  private DragProcessDisplayer dragDisplayer;
  /**
   * This is the DrawingElementFinder of this view.
   */
  private DrawingElementFinder def;

  /**
   * This list defines the Z-order of the managed elements.
   * Elements with higher indices, are on top.
   */
  protected LinkedList elements;


  /**
   * This class constructor.
   * This method just carries over "low level" responsibilities
   * (related to drag, highlight, etc), which favours the
   * extension process of this class.
   */
  public AbstractDrawingElementFinderClient()
  ///////////////////////////////////////////
  {
    this.elements = new LinkedList();

    // -- Initializes a selection.
    this.selection = new DrawingElementSelection();

    // -- Initializes a highlighter.
    this.highlighter = new DrawingElementHighlighter( this );

    // -- Register the DrawingElementFinder (after initializing views).
    def = new DrawingElementFinder(this);

     // -- Initilizes a SelectionDisplayer.
    this.selectionDisplayer = new SelectionDisplayer( this.selection );

    // -- Initilizes a DragProcessDisplayer.
    this.dragDisplayer = new DragProcessDisplayer( this );

    this.selector = new DrawingElementSelector(this);
  }


  /**
   * Adds a DrawingElement object to this panel.
   * @param de the object to be added
   */
  public void addDrawingElement( DrawingElement de )
  //////////////////////////////////////////////////
  {
     this.elements.addLast( correctPosition(de) );
  }

  /**
   * Removes a DrawingElement object of this panel.
   * @param de the object to be removed
   * @return <b> true</b> <code>de</code> is in this panel,
   * and <b>false</b> otherwise.
   */
  public boolean removeDrawingElement( DrawingElement de )
  ////////////////////////////////////////////////////////
  {
    return this.elements.remove( de );
  }

  /**
   * Makes possible the iteration through the DrawingElement objects of
   * this class.
   * @return an <code>Iterator</code> object
   */
  public Iterator getDrawingElements()
  ////////////////////////////////////
  {
    return this.elements.iterator();
  }

  /**
   * Removes all elements
   */
  public void deleteElements(){
      this.elements.clear();
  }
  /**
   * Default implementation that does nothing.
   *
   * @param g2
   */
  public void doPrePaintComponent( Graphics2D g2 ) {}
  ///////////////////////////////////////////////////


  /**
   * Paint this component.
   * @param g this is where this component will be painted on
   */
  public void paintComponent( Graphics g )
  ////////////////////////////////////////
  {
    Graphics2D g2 = ( Graphics2D )g;
    super.paintComponent( g2 );

    // Draw some nice grid.
    drawGrid( g2 );

    // Process concrete class painting necessary BEFORE this.
    doPrePaintComponent( g2 );

    // -- Display Types
    //    As are painted from the first one to the last, the Z-order is
    //    respected: the last ones are on top.
    Iterator it = this.elements.iterator();
    while ( it.hasNext() )
    {
      DrawingElement de = (DrawingElement) it.next();

      // -- Paint the box.
      de.paintOn( g2 );

      // -- Highlight the current one. Should be here to respect the z-order.
      if ( this.highlighter.isHighlighting( de ) )
        this.highlighter.paintOn( g2 );

      // -- Show the selection. Should be here to respect the z-order.
      if ( this.selection.isSelected( de ) )
        this.selectionDisplayer.drawSelectionOn( g2, de );

    } // while

    if(this.selector.isSelecting())
      selector.drawSelectionOn(g2);
  }

  /**
   * Draws a grid at this panel.
   * @param g this is where this panel is being painted on
   */
  private void drawGrid( Graphics2D g )
  /////////////////////////////////////
  {
    final int deltaX = 15;
    final int deltaY = 15;
    g.setColor( Color.gray );

    // -- Paint points.
    Dimension dim = getSize();
    for ( int x = 0; x < dim.width; x += deltaX )
    {
      for ( int y = 0; y < dim.height; y += deltaY )
      {
        if ( x % (deltaX*4) == 0  &&  y % (deltaY*4) == 0 )
          g.fillRect( x, y, 2, 2 );
        else
          g.drawLine( x, y, x, y );
      }
    }
  }

  /**
   * Selects all the DrawingElement objects of this class.
   */
  public void selectAll()
  ///////////////////////
  {
    this.selection.clearSelection();

    for ( Iterator it = this.elements.iterator(); it.hasNext(); )
      this.selection.toggleSelectionStateFor( (DrawingElement) it.next() );
    repaint();
  }

  /**
   * Clear the selection of this class (if there is one).
   */
  public void clearSelection()
  ////////////////////////////
  {
    this.selection.clearSelection();
    repaint();
  }

  /**
   * @return the selection of the DrawingElement objects of this class.
   */
  public DrawingElementSelection getSelection()
  /////////////////////////////////////////////
  {
    return this.selection;
  }



  //
  // -- DrawingElementFinderClient interface.
  //

  /**
   * Tells this view that one of its DrawingElement objects
   * has been clicked on.
   * @param de the clicked on object
   * @param e the click event
   */
  public void drawingElementClicked( DrawingElement de, MouseEvent e )
  ////////////////////////////////////////////////////////////////////
  {
    if ( de != null )
    {
      if ( e.getClickCount() == 1 )
      {
        if ( de != this.selection )
        {
          // Puts the clicked element on top.
          bringToTop( de );
          this.selection.toggleSelectionStateFor( de );
        } // if de
      } // if getClickCount() == 1
    }
    else

      // Clicked over no element. Unselect all.
	this.selection.clearSelection();

    repaint();
  }

  public void startDragging(MouseEvent e){

    	this.selection.clearSelection();
      if((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0)
        this.selector.startSelection(e.getPoint());
  }

  /**
   * Changes the z-order of de, puting it on the top off all other elements.
   * @param de is the object whose order will be changed
   */
  public void bringToTop( DrawingElement de )
  ///////////////////////////////////////////
  {
    boolean ok = this.elements.remove( de );

    if ( !ok )
      throw new RuntimeException( "** UnexpectedError: can't find DrawingElement." );
    else
    {
      this.elements.add( de );
    }
  }

  /* Tells this view that one of its DrawingElement objects
   * has been flyed over.
   * @param de the clicked on object
   */
  public void drawingElementFliedOver( DrawingElement de )
  ////////////////////////////////////////////////////////
  {
    // -- de may be null.
    this.highlighter.setClient( de );
  }

  /**
   * Tells to the view that one of its DrawingElement objects
   * has been dragged.
   * @param de the dragged object
   */
  public void drawingElementDragged( DrawingElement de, MouseEvent e )
  //////////////////////////////////////////////////////
  {

    if ( de != null )
    {
      // -- de belongs to current selection?
      if ( this.selection.isSelected( de ) )
      {
        // -- Yes. So, all the selection will be processed.
        // -- Delegate to displayer.
        this.dragDisplayer.drawingElementDragged( correctPosition(this.selection) );
      } // if
      else
      {
        // -- Delegate to displayer.
        this.dragDisplayer.drawingElementDragged( correctPosition(de) );
      } // else

    } // if
    else{
      selector.keepSelecting(e.getPoint());
      repaint(/*selector.bounds()*/);
    }
      ; // nothing to do.

  }

  /**
   * Finalizes the dragging process (if there is one).
   */
  public void drawingElementEndDrag()
  ///////////////////////////////////
  {
    this.dragDisplayer.drawingElementEndDrag();
    if(selector.isSelecting()){
      selector.endSelection();
    }
  }


  public DrawingElementContainer getContainer(){
     return (new SelectionAwareDrawingElemenContainer( this.elements, this.selection ));
  }

  /**
   * @return the DrawingElementHighlighter object of this view.
   */
    public DrawingElementHighlighter getHighlighter(){
      return  highlighter;
    }

  /**
   * Sets the selection displayer mode as filled.
   * @param color the selection color to be displayed
   */
    public void filledSelection(Color color){
      selectionDisplayer.filledSelection(color);

    }

    /**
     * Sets the selection displayer mode as dashed.
     * This is the default.
     */
    public void dashedSelection(){
      selectionDisplayer.dashedSelection();
    }

  /**
   * @return the DrawingElementFinder of this view
   */
    public DrawingElementFinder getDrawingElementFinder(){
      return def;
    }

   protected DrawingElement correctPosition(DrawingElement dragged){
   double x = dragged.getBounds().getX(),
        y = dragged.getBounds().getY(),
        maxX = this.getWidth() - dragged.getBounds().getWidth(),
        maxY = this.getHeight() - dragged.getBounds().getHeight();


    if(x < 0) dragged.setPosition(0, y);
    else if(x  > maxX) dragged.setPosition(maxX, y);

    if(y < 0) dragged.setPosition(dragged.getBounds().getX(), 0);
    else if( y > maxY) dragged.setPosition(dragged.getBounds().getX(), maxY);

    return dragged;
  }

  public void selectAll(Set des){

    this.selection.clearSelection();

    for ( Iterator it = des.iterator(); it.hasNext(); ){
      DrawingElement de = (DrawingElement) it.next();
      if(de != this.selection){
        bringToTop(de);
        this.selection.addToSelection(de);
      }
    }
   repaint();
  }

  protected void select(DrawingElement de){
    selection.addToSelection(de);
  }


} // class DrawingPane


