package br.usp.ime.coed.DEFSupport;

import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;

/**
 * Responsible for listening to mouse events in the usual way.
 * It is, this class do what every graphical program does:
 * dragging, selecting, etc.
 */
public class DefaultDEFBehaviour  implements DEFBehaviour
/////////////////////////////////////////////////////////
{


  /**
   * The element being currently dragged or null if no one.
   *
   * @supplierRole draggedElement
   * @supplier Cardinality 0..1
   */
  private DrawingElement draggedElement;
  /**
   * This one marks the initial drag point when there is a
   * dragging process ocurring.
   */
  private Point2D initialDragPoint;
  /**
   * This one marks the initial position of the dragged element.
   */
  private Point2D initialElementDragPosition;

  /**
   * Element currently being flied-over or null if no one.   *
   * @supplierRole fliedoverElement
   * @supplierCardinality 0..1
   */
  private DrawingElement fliedOverElement;
  /**
   * The object that contains the DrawingElements.
   * @supplierCardinality 1
   * @supplier Role container
   */
  private DrawingElementContainer container;

  /**
   * The view of the DrawingElementFinder which has this
   * kind of behaviour.
   */
  private AbstractDrawingElementFinderClient client;


  /**
   * Initializes an instance.
   *
   * @param view    Who will be listened for events and notified of those
   *                  processed events.
   * @param container Who knows the elements interested in being queried about
   *                  when a mouse event occurs.
   */
  public DefaultDEFBehaviour( AbstractDrawingElementFinderClient client )
  ////////////////////////////////////////////////////////////////
  {
    // -- Sets initial state.
    this.client = client;
    container = client.getContainer();
    // -- No dragging initially.
    this.draggedElement             = null;
    this.initialElementDragPosition = new Point2D.Float();
    // -- Formats the highlighting process.
    client.getHighlighter().setHighlight(Color.red, 2.0F);
  }

  //
  // -- MouseListener interface.
  //

  /**
   * Mouse listener interface implementation.
   * Invoked when the mouse has been clicked on the view.
   * Will notify the view the DrawingElement under the mouse that has
   * higher Z-order. If no one is under the mouse will notify null.
   *
   * @param e
   */
  public void mouseClicked( MouseEvent e )
  ////////////////////////////////////////
  {
    // -- Notify view.
    DrawingElement de = container.searchElementAt( e.getPoint() );
    client.drawingElementClicked( de, e );
  }

  /**
   * Mouse listener interface implementation.
   *
   * @param e
   */
  public void mousePressed( MouseEvent e )
  ////////////////////////////////////////
  {
    if ( draggedElement == null ) // already dragging?
    { // Not yet in a dragging.
      Point2D  currentPoint = e.getPoint();
      draggedElement = container.searchElementAt( currentPoint );
      if ( draggedElement != null ) // draging over element?
      {
        // Start of new drag.
        initialDragPoint = currentPoint;
        initialElementDragPosition.setLocation( draggedElement.getX(),
                                                draggedElement.getY() );
      } // if
      else
        client.startDragging(e); // Dragging over nothing. In the future, draw rubberband.
    } // if
    else
      throw new RuntimeException(
      "** Unexpected error: mousePressed when already exist a draggedElement." );

  }

  /**
   * Mouse listener interface implementation.
   * Does nothing.
   * @param e
   */
  public void mouseEntered( MouseEvent e ){}
  ////////////////////////////////////////

  /**
   * Mouse listener interface implementation.
   * Does nothing.
   * @param e
   */
  public void mouseExited( MouseEvent e ) {}
  ///////////////////////////////////////

  /**
   * Mouse listener interface implementation.
   * @param e
   */
  public void mouseReleased( MouseEvent e )
  /////////////////////////////////////////
  {
  //  if ( draggedElement != null )
  //  {
      // -- End of dragging.
      draggedElement = null;
      initialDragPoint = null;
      // -- Notify view.
      client.drawingElementEndDrag();
  //  }
  //  else view.

  }

  //
  // -- MouseMotionListener interface.
  //

  /**
   * Invoked when a mouse button is pressed on a component and then
   * dragged.  Mouse drag events will continue to be delivered to
   * the component where the first originated until the mouse button is
   * released (regardless of whether the mouse position is within the
   * bounds of the component).
   */
  public void mouseDragged( MouseEvent e )
  ////////////////////////////////////////
  {
    Point2D  currentPoint = e.getPoint();

    if ( draggedElement == null ) // already dragging?
    {
      client.drawingElementDragged(null, e);// Dragging over nothing. In the future, draw rubberband.
    } // if
    else
    { // -- Already dragging.
      //    Calculate drag distance.
      draggedElement.setPosition(
      initialElementDragPosition.getX() + currentPoint.getX() - initialDragPoint.getX(),
      initialElementDragPosition.getY() + currentPoint.getY() - initialDragPoint.getY() );
      // -- Notify view.
      client.drawingElementDragged( draggedElement, e );
    } // else
  }

  /**
   * Highlight any DrawingElement object if the mouse
   * moves on it.
   * @param e
   */
  public void mouseMoved( MouseEvent e )
  //////////////////////////////////////
  {
    DrawingElement currentDE = container.searchElementAt( e.getPoint() );

    if ( currentDE == null ) // some element under cursor?
    {
      if ( fliedOverElement != null )
      {
        fliedOverElement = null;
      }
      else // fliedOverElement == null
            return;
    }
    else // currentDE != null
    {
      if ( fliedOverElement == null )
      {
        fliedOverElement = currentDE;
      }
      else // fliedOverElement != null
      {
        if ( fliedOverElement != currentDE )
        {
          fliedOverElement = currentDE;
        }
        else // fliedOverElement == currentDE
              return;
      } // else
    } // else

    client.drawingElementFliedOver( fliedOverElement );
  }

} //  class DrawingElementFinder







