package br.usp.ime.coed.DEFSupport;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.geom.Rectangle2D;

import javax.swing.JComponent;

/**
 * Manages the highlighting of DrawingElements.
 * Only one de can be highlighted (HL) simultaneously.
 * When notified of a change in the HL element, will interact with the view
 * component, and ask to him to perform the minimum necessary repaints.
 */
public class DrawingElementHighlighter
//////////////////////////////////////
{
  /**
   * The highlighted drawing element. May be null.
   * @supplierCardinality 0..1
   * @supplierRole view
   */
  private DrawingElement client;
  /**
   * The highlighting color.
   */
  private Color flyOverColor;
  /**
   * Keeps in memory the last view.
   * @supplierCardinality 0..1
   * @supplierRole lastHighlighted
   */
  private DrawingElement lastHighlighted = null;
  /**
   * Sets the way highlight will be drawn.
   */
  private Stroke flyOverStroke =
    new BasicStroke ( 2.0F, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL );

  /**
   * This is panel that uses this highlighting class.
   */
  private JComponent clientComponent;

  /**
   * Constructor. Sets the highlight color as red.
   * @param clientComponent the component that will use this class
   * to highlight any object it may have.
   */
  public DrawingElementHighlighter( JComponent clientComponent )
  //////////////////////////////////////////////////////////////
  {
    this(clientComponent, Color.red);
  }

  /**
   * Constructor.
   * @param clientComponent the component that will use this class to
   * highlight its objects.
   * @param color the highlight color.
   */
   public DrawingElementHighlighter( JComponent clientComponent, Color color )
  //////////////////////////////////////////////////////////////
  {
    this.clientComponent = clientComponent;
    flyOverColor = color;
  }

  /**
   * Constructor.
   * @param clientComponent the component that will use this class to
   * highlight its objects.
   * @param color the highlight color.
   * @param width the highlight line width.
   */
  public DrawingElementHighlighter( JComponent clientComponent, Color color, float width )
  //////////////////////////////////////////////////////////////
  {
    this.clientComponent = clientComponent;
    setHighlight( color, width );
  }
  /**
   * Changes the current view (the one to be highlighted).
   * @param view the new view
   */
  public void setClient( DrawingElement client )
  //////////////////////////////////////////////
  {
    // -- Update state.
    lastHighlighted = this.client;
    this.client = client;

    // -- Ask for repainting of the changed zone:
    //     - the old HL, now without HL.
    //     - the new HL.
    if ( lastHighlighted != null )
    {
      clientComponent.repaint( getClipBoundsFor( lastHighlighted.getBounds() ) );
    }
    if ( client != null )
    {
      clientComponent.repaint( getClipBoundsFor( client.getBounds() ) );
    }
  }


  /**
   * Informs if has some registered view.
   */
  public boolean hasClient()
  //////////////////////////
  {
    return client != null;
  }


  /**
   * Informs if the currently highlighted element is view.
   * @param view the element that may be the current view.
   */
  public boolean isHighlighting( DrawingElement client )
  //////////////////////////////////////////////////////
  {
    return this.client == client;
  }

  /**
   * If there is any view, return a non null bound.
   * @return a <code>Rectangle2D</code> object denoting the
   * bounds of the highlighted area (if there is one).
   */
  public Rectangle2D getClipBounds()
  //////////////////////////////////
  {
    Rectangle2D clipBounds = null;
    if ( client != null )
    {
      clipBounds = getClipBoundsFor( client.getBounds() );
    }
    return clipBounds;
  }

  /**
   * Tells what is the highlighted area for the bounds
   * of a view.
   * @param bounds the bounds of a view.
   * @return the highlighted area.
   */
  public static Rectangle getClipBoundsFor( Rectangle2D bounds )
  //////////////////////////////////////////////////////////////
  {
    Rectangle clipBounds =
      new Rectangle( (int)bounds.getX() - 20, (int)bounds.getY() - 20,
                     (int)bounds.getWidth() + 40, (int)bounds.getHeight() + 40 );
    return clipBounds;
  }

  /**
   * Paints the highlighting of a view.
   * @param g this is where the painting will be made on.
   */
  public void paintOn( Graphics2D g )
  //////////////////////////////////
  {
    if ( client != null )
    {
      Rectangle2D bounds = client.getBounds();
      g.setColor( flyOverColor );
      Stroke oldStroke = g.getStroke();
      g.setStroke( flyOverStroke );
      g.draw( new Rectangle2D.Double( bounds.getX() - 2, bounds.getY() - 2,
                                      bounds.getWidth() + 3 + 1, bounds.getHeight() + 3 + 1) );


      g.setStroke( oldStroke );
    }
  }

  /**
   * Configures the highlighting picture.
   * @param color the new highlighting color.
   * @param width the new highlighting line width.
   */
  public void setHighlight(Color color, float width){
    flyOverColor = color;
    flyOverStroke = new BasicStroke ( width, BasicStroke.CAP_ROUND,
                                           BasicStroke.JOIN_BEVEL );
  }

} // class DrawingElementHighlighter
