package br.usp.ime.coed.DEFSupport;

import java.awt.Graphics2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;


/**
 * DrawingElement that manages a set of other DrawingElements.
 * Represents a user selection.
 *
 * @pattern composite.
 */
public class DrawingElementSelection
  implements DrawingElement
////////////////////////////////////
{
  private Set selectionListeners;

  /**
   * Collection od DrawingElements currently selected.
   * Uses a Set to ensure that there are no repeated elements.
   * Uses the hashValue() of each element, so it should be something ok.
   *
   * @associates <{DrawingElement}>
   * @supplier Cardinality 0..
   * @supplier Role selectedElements
   */
  private Set selectedElements;

  public DrawingElementSelection()
  ////////////////////////////////
  {
    this.selectedElements = new HashSet();
    selectionListeners = new HashSet();
  }


  /**
   * Toggles the selection state of the specified element.
   */
  public void toggleSelectionStateFor( DrawingElement de )
  ////////////////////////////////////////////////////////
  {
    if ( selectedElements.contains( de ) )
      selectedElements.remove( de );
    else
      selectedElements.add( de );
    fireSelectionEvent();

  }


  /**
   * Adds de to the selection, if not already there.
   */
  public void addToSelection( DrawingElement de )
  ///////////////////////////////////////////////
  {
    selectedElements.add( de );
    fireSelectionEvent();
  }


  /**
   * Removes de from the selection, if already selected.
   */
  public void removeFromSelection( DrawingElement de )
  ////////////////////////////////////////////////////
  {
    selectedElements.remove( de );
    fireSelectionEvent();
  }


  /**
   * Removes all elements from the selection.
   */
  public void clearSelection()
  ////////////////////////////
  {
    selectedElements.clear();
    fireSelectionEvent();
  }


  /**
   * Clears the selection and adds de initiating a new selection.
   */
  public void setAsNewSelection( DrawingElement de )
  //////////////////////////////////////////////////
  {
    this.clearSelection();
    this.addToSelection( de );
    fireSelectionEvent();
  }


  /**
   * Informs if de is currently selected.
   */
  public boolean isSelected( DrawingElement de )
  //////////////////////////////////////////////
  {
    return selectedElements.contains( de );
  }


  /**
   * Returns a set holding the selected elements.
   * This is an unmodifiable collection.
   */
  public Set getSelectedElements()
  ////////////////////////////////
  {
    return new HashSet/*Collections.unmodifiableSet*/( selectedElements );
  }


  //
  // -- Interface DrawingElement
  //

  /**
   * Returns the x position of the selection.
   * Id est, the minimum x in the selected objects or 0 if none.
   *
   * @todo: optimize caching the result of getX() and getY().
   *        Every method that changes the state,
   *        sets as dirty that cache and imposes a recalculation.
   */
  public double getX()
  ////////////////////
  {
    boolean changed = false;
    double minX = Double.MAX_VALUE; // default value.
    for ( Iterator it = selectedElements.iterator(); it.hasNext(); )
    {
      DrawingElement de = (DrawingElement)it.next();
      if ( de.getX() < minX )
      {
        minX = de.getX();
        changed = true;
      }
    } // for

    return (changed ? minX : 0);
  }


  /**
   * Returns the y position of the selection.
   * Id est, the minimum y in the selected objects or 0 if none.
   *
   * @return
   */
  public double getY()
  ////////////////////
  {
    boolean changed = false;
    double minY = Double.MAX_VALUE; // default value.
    for ( Iterator it = selectedElements.iterator(); it.hasNext(); )
    {
      DrawingElement de = (DrawingElement)it.next();
      if ( de.getY() < minY )
      {
        minY = de.getY();
        changed = true;
      }
    } // for
    return (changed ? minY : 0);
  }


  /**
   * Sets the position of its contained elements.
   * The position of each component will be the offset from the result of
   * applying this.getX() and this.getY().
   *
   * @param x
   * @param y
   */
  public void setPosition( double x, double y )
  /////////////////////////////////////////////
  {
    double offsetX = x - this.getX();
    double offsetY = y - this.getY();

    for ( Iterator it = selectedElements.iterator(); it.hasNext(); )
    {
      DrawingElement de = (DrawingElement)it.next();
      de.setPosition( de.getX() + offsetX, de.getY() + offsetY );
    } // for
  }


  /**
   * Returns the minimal Rectangle enclosing its geometry.
   *
   * @return
   */
  public Rectangle2D getBounds()
  //////////////////////////////
  {
    Point2D max = getMaxPoint();
    double minX = getX();
    double minY = getY();
    return new Rectangle2D.Double( minX, minY, max.getX() - minX, max.getY() - minY );
  }


  /**
   * Returns the point belonging to any of its elements that has maximum
   * x and y value.
   */
   private Point2D getMaxPoint()
  //////////////////////////////
  {
    double maxX = 0; // default value.
    double maxY = 0; // default value.
    for ( Iterator it = selectedElements.iterator(); it.hasNext(); )
    {
      DrawingElement de = (DrawingElement)it.next();
      double currVal = de.getX() + de.getBounds().getWidth();
      if ( maxX < currVal )
        maxX = currVal;

      currVal = de.getY() + de.getBounds().getHeight();
      if ( maxY < currVal )
        maxY = currVal;
    } // for
    return new Point2D.Double( maxX, maxY );
  }


  /**
   * Informs if the geometry of thid object includes the specified point,
   * i.e., if any of its elements include it.
   */
  public boolean contains( Point2D p )
  ////////////////////////////////////
  {
    boolean found = false; // default value.
    for ( Iterator it = selectedElements.iterator(); it.hasNext(); )
    {
      DrawingElement de = (DrawingElement)it.next();
      if ( de.contains( p ) )
      {
        found = true;
        break;
      }
    } // for
    return found;
  }


  public void paintOn( Graphics2D g )
  ///////////////////////////////////
  {
    // nothing to do.
    // @todo: join with SelectionDisplayer???
  }

  public boolean isEmpty(){
    return selectedElements.isEmpty();
  }


  public void addSelectionListener(SelectionListener listener){
    this.selectionListeners.add(listener);
  }

  public void removeSelectionListener(SelectionListener listener){
    this.selectionListeners.remove(listener);
  }

  private void fireSelectionEvent(){
    SelectionEvent event = new SelectionEvent(this, isEmpty());
    Iterator listeners = selectionListeners.iterator();
    while(listeners.hasNext())
      ((SelectionListener)listeners.next()).selectionChanged(event);
  }

} // class DrawingElementSelector
